#!/usr/bin/perl
#
# fraglets0.18/bin/mutate.pl
#
# mutation operator (intended to replace mutate.c)
#
# Copyright (C) 2005 <lidia.yamamoto@unibas.ch>
#
# This file is distributed under the terms of the GNU general Public
# License (GPL), see the file COPYING
#
# usage: mutate <endmarker>
#
# input (stdin) is a genotype with mutation instructions
# output (stdout) is the mutated genotype
#
# endmarker marks the end of the genotype (mutation will not occur beyond
# that)
#
# for more info on mutation instruction format, see ../doc/mutate_doc.txt
#
# History:
# 20050221 bug fixes (fraglets v0.18)
# 20050131 rewritten from mutate.c by lidia.yamamoto@unibas.ch
#

$endmarker = shift(@ARGV);

sub parse_frag {
    local($node,$fcore,$mult) = @_;
    $mult = 1 if ($mult eq "");
    if ($mult > 0 && $maxfc < 0) {
	local(@symlist) = split(/:/, $fcore);
	local($nsyms) = $#symlist + 1;
	#print "node=$node fcore=<$fcore> mult=$mult nsyms=$nsyms\n";
	$frec[$fc]{node} = $node;
	@{ $frec[$fc]{symlist} } = @symlist;
	$frec[$fc]{nsyms} = $nsyms;
	$frec[$fc]{mult} = $mult;
	$tnsyms += $nsyms * $mult;
    }

    # should mult itself counts as one symbol?
    # $tnsyms += $nsyms * $mult + 1;
}

sub parse {
    while (<>) {
	if ( /^g[ \t]+\/(.*)\/[ \t]*$/ ) {
	    $gencore=$1;
	    @flist = split(/\//, $gencore);
	    $fc = 0;
	    $tnsyms = 0;
	    $maxfc = -1;
	    foreach $frag (@flist) {
		#print "frag=$frag\n";
		if ($frag =~  /(.*)\[(.*)\]([0-9]*)/ ) {
		    parse_frag($1,$2,$3);
		} elsif ($frag =~ /^m[ \t]+(.*)[ \t]*/) {
		    $marker=$1;
		    #print "marker=$marker\n";
		    if ($endmarker ne "" && $marker eq $endmarker) {
			$maxfc = $fc;
		    }
		}
		$fc ++;
	    }
	    if ($maxfc == -1) {
		$maxfc = $fc;
	    }
	}
	if ( /^([0-9]+)[ \t]+(.*)[ \t]*$/ ) {
	    local($prob) = $1;
	    local($sym) = $2;
	    if ($prob > 0) {
		$rprob[$ic]=$prob;
		$rsym[$ic]=$sym;
		$ic++;
		$rptot += $prob;
	    }
	}
    }
}

sub new_symbol {
    $dice = int (rand() * ($rptot));

    $cumm = 0;
    for (local($i)=0; $i < $ic; $i++) {
	#print "rprob=<$p>, rsym=<$s>\n";
	$cumm += $rprob[$i];
	if ($dice <= $cumm) {
	    return $i;
	}
    }
    #print "rptot=$rptot tnsyms=$tnsyms\n";
    return $i-1;
}

sub print_genotype {
    print "g /";
    for ($i=0; $i < $fc; $i++) {
	print "$flist[$i]/";
	#print "flist[$i]=$flist[$i]\n";
    }
    print "\n";
}


sub mutate {
    $target = int (rand() * ($tnsyms-1));
    #print "target=$target\n";

    $searched = 0;
    for (local($i)=0; $i < $maxfc; $i++) {
	next if ($flist[$i] =~ /^m[ \t]+/);
	$nsyms = $frec[$i]{nsyms};
	$mult = $frec[$i]{mult};
	$ms = $nsyms * $mult;
	if ($searched + $ms > $target) {
	    $ts = ( $target - $searched ) % $nsyms ;
	    #print "to patch=$flist[$i] at post=$ts sym=${ $frec[$i]{symlist} }[$ts] \n";
	    $oldsym =${ $frec[$i]{symlist} }[$ts];
	    while ( ($newsym = $rsym[new_symbol()]) eq $oldsym) {};
	    #$newsym = $rsym[new_symbol()];

	    local($mfrag)="";
	    if ($mult > 1) {
		# mutate only one fraglet of pool, preserve mult-1 others
		local($fcore)= join(':', @{ $frec[$i]{symlist} });
		local($mult) = $frec[$i]{mult} - 1;
		$mfrag = "/$frec[$i]{node}\[$fcore\]$mult";
	    }
	    ${ $frec[$i]{symlist} }[$ts] = $newsym;
	    local($fcore) = join(':', @{ $frec[$i]{symlist} });
	    $flist[$i] = "$frec[$i]{node}\[$fcore\]1$mfrag";
	    #print "patched=$flist[$i]\n";
	    $i = $fc; # should be 'break'
	}
	$searched += $ms;
    }
}


$ic=0;
$fc=0;
$rptot=0;
&parse();

if ($ic > 0 && $fc > 0) {
    &mutate();
    &print_genotype();
} else {
    print STDERR "mutate.pl: wrong input format\n";
}

