#!/usr/bin/perl
#
# fraglets0.23/bin/concentr.pl
#
# concentration parser: takes the output of the fraglet interpreter
# and produces the concentration of the given fraglet as a function of
# time (in execution steps)
#
# Copyright (C) 2006 Lidia Yamamoto <lidia.yamamoto@unibas.ch>
#
# This file is distributed under the terms of the GNU general Public
# License (GPL), see the file COPYING
#
# History:
# 20070110 modified for dynamic evolution experiments
# 20061109 created by lidia.yamamoto@unibas.ch
#

$usage="Usage: concentr.pl <n> [<interv>] \n  where <n> is the id of the rule searched,\n  and <interv> is the average interval between samples\n";

$nrule= shift(@ARGV);
die "$usage" if ($nrule eq "");

$interv=shift(@ARGV);
$interv=0 if ($interv eq "");
die "$usage" if ($interv < 0);


$sfrag="";

$sfrag='\[a'              if ($nrule eq "a");
$sfrag='\[food'           if ($nrule eq "food");
$sfrag='\[poison'         if ($nrule eq "poison");
$sfrag='\[gene match'     if ($nrule eq "gene");
$sfrag='\[do something'   if ($nrule eq "loop");

$sfrag='r\[exp1'            if ($nrule eq "exp1");
$sfrag='r\[exp2'            if ($nrule eq "exp2");
$sfrag='r\[exp3'            if ($nrule eq "exp3");
$sfrag='a\[match serv .*1.*' if ($nrule eq "ms1");
$sfrag='a\[match serv .*2.*' if ($nrule eq "ms2");
$sfrag='a\[match serv .*3.*' if ($nrule eq "ms3");
$sfrag='a\[serv'            if ($nrule eq "serv");

die "Unknown rule \"$nrule\"\n$usage" if ($sfrag eq "");

$time = 0;
$ptime = 0;
$c0 = 0;
$lastcdiff = 0;

sub parse_frag {
    local($cdiff,$node,$fcore,$mult) = @_;
    $mult=1 if ($mult == "");
    if ("$node\[$fcore" =~ /$sfrag/) {
	#print "FOUND: $node\[$fcore\]$mult\n";
	if ($mult > 0) {
	    if ($cdiff > 0) {
		$c0 += $mult;
	    } elsif ($cdiff < 0) {
		$c0 -= $mult;
	    } else {
		# absolute concentration: add multiplicies if multiple
		# matchin rules, e.g.:
		# [match:a:do:something]8
                # [match:a:do:someth:different]6
		# total concentration of [match:a] = 8+6=14
		if ($lastcdiff == $cdiff) {
		    $c0 += $mult;
		} else {
		    $c0 = $mult;
		}
	    }

	    #print "time=$time c0=$c0\n";

	    #if ($time >= $ptime+$interv) {
	    print "$time $c0\n";
	    $ptime = $time;
	    #}
	    $lastcdiff = $cdiff;
	}
    } else {
	#print "Not found: $node\[$fcore\]$mult\n";
    }
}

sub parse {
    while (<>) {
	if ( /^f +(.*)\[(.*)\]([0-9]*)/ ) {
	    parse_frag(0,$1,$2,$3);
	}
	if ( /^([0-9]+) +(.+) *\-\-\> +(.+)$/ ) {
	    $time=$1;
	    $educts=$2;
	    $prodts=$3;

	    @elist = split(/\,/, $educts);
	    @plist = split(/\,/, $prodts);

	    #print "time=$time\n";
	    #print "educts=<@elist>\n";
	    #print "prodts=<@plist>\n\n";

	    foreach $frag (@elist) {
		#print "efrag=$frag\n";
		if ($frag =~  /(.*)\[(.*)\]([0-9]*)/ ) {
		    parse_frag(-1,$1,$2,$3);
		}
	    }
	    foreach $frag (@plist) {
		#print "pfrag=$frag\n";
		if ($frag =~  /(.*)\[(.*)\]([0-9]*)/ ) {
		    parse_frag(1,$1,$2,$3);
		}
	    }
	    #print "\n";
	}

	if ($interv > 0 && $time >= $ptime + $interv) {
	    print "$time $c0\n";
	    $ptime = $time;
	}
    }
}


&parse();

# guarantees at least two points in file so that it can be plotted:
if ($ptime == 0) {
    # empty file, print at least the starting point
    print "$time $c0\n";
}
if ($ptime < $time) {
    # print an end point if not done yet
    print "$time $c0\n";
}

