#!/usr/bin/perl
#
# fraglets0.23/bin/rate.pl
#
# calculates the rate of rules (events such as activation/inhibition
# signals) in events per simulation step, for every $interv simulation steps
#
# Copyright (C) 2006 Lidia Yamamoto <lidia.yamamoto@unibas.ch>
#
# This file is distributed under the terms of the GNU general Public
# License (GPL), see the file COPYING
#
# History:
# 20070108 print average rate to stderr
# 20061109 created by lidia.yamamoto@unibas.ch
#

$usage="Usage: rate.pl e|p|r <n>\n  where:\n  p: production rate\n  e: consumption (educt) rate\n  r: p minus e\n  <n> is the id of the rule searched\n";

# rate="p" for production rate, "e" for (educt) consumption, "r" for p-e
# rate="c" for absolute concentration (not actually rate)
$rate= shift(@ARGV);
die "$usage" if ($rate eq "");
# rate every $interv simulation steps

$nrule= shift(@ARGV);
die "$usage" if ($nrule eq "");

#print "RATE=$rate\nRULE=$nrule\n";

$interv=100; 

$sfrag="";
$sfrag='a\[match\:a\:.*1.*' if ($nrule eq "ma1");
$sfrag='a\[match\:a\:.*2.*' if ($nrule eq "ma2");
$sfrag='a\[match\:data\:.*1.*' if ($nrule eq "mdata1");
$sfrag='a\[match\:serv\:.*1.*' if ($nrule eq "ms1");
$sfrag='a\[match\:serv\:.*2.*' if ($nrule eq "ms2");
$sfrag='a\[match\:serv\:.*3.*' if ($nrule eq "ms3");
$sfrag='a\[a'               if ($nrule eq "a");
$sfrag='a\[food'            if ($nrule eq "food");
$sfrag='a\[serv'            if ($nrule eq "serv");
$sfrag='r\[exp1'            if ($nrule eq "exp1");
$sfrag='r\[exp2'            if ($nrule eq "exp2");
$sfrag='r\[exp3'            if ($nrule eq "exp3");
$sfrag='r\[reward'          if ($nrule eq "reward");
$sfrag='r\[reward1'         if ($nrule eq "reward1");
$sfrag='r\[reward2'         if ($nrule eq "reward2");
$sfrag='r\[reward3'         if ($nrule eq "reward3");
$sfrag='r\[punish1'         if ($nrule eq "punish1");
$sfrag='r\[punish2'         if ($nrule eq "punish2");
$sfrag='r\[punish3'         if ($nrule eq "punish3");

die "Unknown rule \"$nrule\"\n$usage" if ($sfrag eq "");

$time = 0;  # last time a reaction (of any kind) was seen
$stime = 0; # last time a reaction of the searched type was seen
$ptime = 0; # last printed time
$c0 = 0;    # concentration observed within $interv
$total = 0; # total concentration

sub parse_frag {
    local($cdiff,$node,$fcore,$mult) = @_;
    $mult=1 if ($mult == "");
    if ("$node\[$fcore" =~ /$sfrag/ && $mult > 0) {
	$c0 += $cdiff * $mult;
	$total += $cdiff * $mult;
	$stime = $time;
	if ($stime >= $ptime+$interv) {
	    $r = $c0 / ($stime - $ptime);
	    print "$stime $r\n";
	    $c0 = 0;
	    $ptime = $stime;
	}
    }
}

sub parse {
    while (<>) {
	if ( /^([0-9]+) +(.+) *\-\-\> +(.+)$/ ) {
	    $time=$1;
	    $educts=$2;
	    $prodts=$3;

	    if ($rate eq "e" || $rate eq "r") {
		@elist = split(/\,/, $educts);
		foreach $frag (@elist) {
		    #print "efrag=$frag\n";
		    if ($frag =~  /(.*)\[(.*)\]([0-9]*)/ ) {
			parse_frag(-1,$1,$2,$3);
		    }
		}
	    }

	    if ($rate eq "p" || $rate eq "r") {
		@plist = split(/\,/, $prodts);
		foreach $frag (@plist) {
		    #print "pfrag=$frag\n";
		    if ($frag =~  /(.*)\[(.*)\]([0-9]*)/ ) {
			parse_frag(1,$1,$2,$3);
		    }
		}
	    }
	    #print "\n";
	}
    }
}


&parse();

#print "END ptime=$ptime\n";
# guarantees at least two points in file so that it can be plotted:
if ($ptime == 0) {
    # empty file, print at least the starting point
    print "$time $c0\n";
}
# rate should be zero if no info at the end
if ($ptime+$interv < $time) {
    $r = $c0 / $interv;
    $ptime += $interv;
    print "$ptime $r\n";
    $c0 = 0;
    if ($ptime+$interv < $time) {
	$ptime += $interv;
	print "$ptime 0\n";
    }
}

#print end point if not done yet
if ($ptime < $time) {
    $r = $c0 / ($time - $ptime);
    print "$time $r\n";
}

if ($stime > 0) {
  $average = $total / $stime;
  print STDERR "average ${nrule}_${rate} = $average events/simstep\n";
} else {
  print STDERR "average ${nrule}_${rate} = 0\n";
}

