/* fraglets0.19/src/sockin.c

  socket interface for fraglets

  Copyright (C) 2005 <Jasenko.Zivanov@stud.unibas.ch>

  This file is distributed under the terms of the GNU general Public
  License (GPL), see the file COPYING

  interface between the fraglets execution environment and sockets;
  part of udp socket support for fraglets,
  by Jasenko Zivanov, University of Basel, May 2005

  History:

  20050530  created by Jasenko Zivanov (Unibas) as part of CS221 project
*/


#include <sys/socket.h>
#include <sys/types.h>
#include <netinet/in.h>
#include <netdb.h>
#include <time.h>


#define MAX_FRAG_SEND_BUFFER 4096
#define SOCKET_TIME_INTERVAL_MSECS 0

int get_instruction_count()
{	
	int out = 0;
	struct node* n = nodelist;
	while(n)
	{
		out += n->opcnt;
		n = n->next;
	}
	
	return out;
}



// Prototypes for "udphelpers.c"  :
extern int create_UDP_socket();
extern void set_socket_address( struct sockaddr_in * socket_address, in_addr_t address, int port);
extern void bind_socket(int socket_fd, struct sockaddr_in* address);
extern void check_node_and_port(void* n, int port);
	
typedef struct UDPNode
{
	struct node* node;
	symbol node_symbol;
	struct UDPNode* next;
	int socketFD, port;
	struct sockaddr address;
	
} UDPNode;

socklen_t sockaddr_size = sizeof(struct sockaddr_in);

UDPNode		*output_list_head = 0, *input_list_head = 0;
int 		udp_node_size = sizeof(UDPNode);

fd_set 		all_input_socket_FDs, input_socket_FDs;
int 		max_input_socket_fd = 1;
struct timeval 	socket_time_interval;

void test_send_times(UDPNode* d)
{
	clock_t start, end;
	int i;
	int time_needed;
	char small[3], big[1024];
	memset(small, 'o', 3);
	memset(big, 'o', 1024);
	small[2] = '\0';
	big[1023] = '\0';
	
	start = clock();     
	
	for (i = 0; i < 1000000; i++)	
	sendto(d->socketFD, small, 3, 0, 
		&d->address, sockaddr_size);
		
	end = clock();
	time_needed = (int)(((double) (end - start)) / CLOCKS_PER_SEC);
	printf("time used for 1000000 3-byte packets: %d secs\n", time_needed);
	
	start = clock();     
	
	for (i = 0; i < 1000000; i++)	
	sendto(d->socketFD, big, 1024, 0, 
		&d->address, sockaddr_size);
		
	end = clock();
	time_needed = (int)(((double) (end - start)) / CLOCKS_PER_SEC);
	printf("time used for 1000000 1024-byte packets: %d secs\n", time_needed);
	
}

void add_udp_node_to_list(NodeLocation location, struct node* n, symbol node_sym,
				struct sockaddr_in* addr, int port, int fd)
{
	UDPNode* head;
	
	if (location == Local) head = input_list_head;
	else head = output_list_head;

	if (head)
	{
		UDPNode* new_node = malloc(udp_node_size);
		new_node->next = head;
		new_node->node = n;
		head = new_node;
	}
	else
	{
		head = malloc(udp_node_size);
		head->next = 0;
		head->node = n;
	}

	head->address = *((struct sockaddr*)addr);
	head->port = port;
	head->socketFD = fd;
	head->node_symbol = node_sym;
	n->udp_wrapper = head;

	if (location == Local) input_list_head = head;
	else output_list_head = head;
}

void open_socket_to_remote_node(symbol node, char* address, int port)
{
	struct node* n = node_lookup(node,0);
	check_node_and_port(n,port);
	
	int fd = create_UDP_socket();
	in_addr_t address_binary;
	
	if (!inet_aton(address, &address_binary)){
		fprintf(stderr, "illegal IPv4 address: %s\n", address);
		exit(-1);
	}
	
	struct sockaddr_in complete_address;
	set_socket_address(&complete_address, INADDR_ANY, 0);
	bind_socket(fd, &complete_address);
	
	set_socket_address(&complete_address, address_binary, port);
	add_udp_node_to_list(Remote, n, node, &complete_address, port, fd);
	n->location = Remote;
}

void send_fraglet_through_socket(struct frag *f, struct node *origin, struct node* dest)
{
	UDPNode* d = dest->udp_wrapper;
	
	if (!dest->udp_wrapper)
	{
		fprintf(stderr, 
			"\nnode %s has no active UDP socket!\n", 
			name_lookup(dest->name));
		return;
	}
	int offset = f->offs;
	
	char buffer[MAX_FRAG_SEND_BUFFER] = "[";
	int i;
	for (i = 0; i < f->len; i++) {
		if (i) strcat(buffer, ":");
		strcat(buffer, name_lookup(f->buf->data[f->offs + i]));
	}
	strcat(buffer, "]");
	int length = strlen(buffer) + 1;
	
	sendto(d->socketFD, buffer, length, 0, 
		&d->address, sockaddr_size);
}

void process_incoming_fraglet(char* string, symbol node)
{	
	printf("receiving fraglet: %s\n", string);
	struct frag* f = f_parsefrag(0, string);	
	f->nd = node;
	rx_fraglet(f);
}

void init_sockets()
{
	FD_ZERO(&all_input_socket_FDs);
	FD_ZERO(&input_socket_FDs);
	socket_time_interval.tv_sec = 0;
	socket_time_interval.tv_usec = 1000*SOCKET_TIME_INTERVAL_MSECS;
}

void open_local_socket(symbol node, int port)
{
	struct node* n = node_lookup(node,0);
	check_node_and_port(n,port);
	
	int fd = create_UDP_socket();
	struct sockaddr_in complete_address;
	
	set_socket_address(&complete_address, INADDR_ANY, port);
	bind_socket(fd, &complete_address);
	add_udp_node_to_list(Local, n, node, &complete_address, port, fd);
	FD_SET(fd, &all_input_socket_FDs);
	max_input_socket_fd = fd+2;
}

void listen_on_local_sockets()
{
	input_socket_FDs = all_input_socket_FDs;
	UDPNode* input_node = input_list_head;
	
	int check = select(
		max_input_socket_fd, &input_socket_FDs, 
		0, 0, &socket_time_interval );
		
	if (check < 0) 
	{
		perror("Select");
		exit(-1);
	}
	
	while ( input_node )
	{	
		if (FD_ISSET(input_node->socketFD, &input_socket_FDs))
		{
			char buffer[MAX_FRAG_SEND_BUFFER];
			
			int read = recvfrom( 
				input_node->socketFD, buffer, 
				MAX_FRAG_SEND_BUFFER-1, 0, 
				&input_node->address, &sockaddr_size );
		
			if (read > 0) 
			{
				buffer[read-1] = '\0';
				process_incoming_fraglet(buffer, 
					input_node->node_symbol);
			}
		}
		input_node = input_node->next;
	}
}

#include "udphelpers.c"
